@@ -94,21 +94,88 @@ let get_output1 ?use_bash cmd =
 
 let mk_dirs_opt pr = String.concat " " (List.map (fun s -> pr ^ (Filename.quote s)) !dirs)
 
+exception Not_utf8
+
+let utf8_next s i =
+  let fail () = raise Not_utf8 in
+  let check i =
+    if i >= String.length s then fail ();
+    let n = Char.code s.[i] in
+    if n lsr 6 <> 0b10 then fail () else n
+  in
+  if !i >= String.length s then fail ();
+  match s.[!i] with
+  | '\000'..'\127' as c ->
+      let n = Char.code c in
+      i := !i + 1;
+      n
+  | '\192'..'\223' as c ->
+      let n1 = Char.code c in
+      let n2 = check (!i+1) in
+      let n =
+        ((n1 land 0b11111) lsl 6) lor
+        ((n2 land 0b111111))
+      in
+      i := !i + 2;
+      n
+  | '\224'..'\239' as c ->
+      let n1 = Char.code c in
+      let n2 = check (!i+1) in
+      let n3 = check (!i+2) in
+      let n =
+        ((n1 land 0b1111) lsl 12) lor
+        ((n2 land 0b111111) lsl 6) lor
+        ((n3 land 0b111111))
+      in
+      i := !i + 3;
+      n
+  | '\240'..'\247' as c ->
+      let n1 = Char.code c in
+      let n2 = check (!i+1) in
+      let n3 = check (!i+2) in
+      let n4 = check (!i+3) in
+      let n =
+        ((n1 land 0b111) lsl 18) lor
+        ((n2 land 0b111111) lsl 12) lor
+        ((n3 land 0b111111) lsl 6) lor
+        ((n4 land 0b111111))
+      in
+      i := !i + 4;
+      n
+  | _ ->
+      fail ()
+
+let toutf16 s =
+  let i = ref 0 in
+  let b = Buffer.create (String.length s * 2) in
+  while !i < String.length s do
+    Buffer.add_utf_16le_uchar b (Uchar.unsafe_of_int (utf8_next s i))
+  done;
+  Buffer.contents b
 
 (* Build @responsefile to work around Windows limitations on
    command-line length *)
 let build_diversion lst =
-  let (responsefile, oc) = open_temp_file "camlresp" "" in
-  List.iter
-    (fun f ->
-      if f <> "" then begin
+  let responsefile = temp_file "camlresp" "" in
+  let oc = open_out_bin responsefile in
+  let lst =
+    List.map (fun f ->
         let s = Bytes.of_string (Filename.quote f) in
         for i = 0 to Bytes.length s - 1 do
           if Bytes.get s i = '\\' then Bytes.set s i '/'
         done;
-        output_bytes oc s; output_char oc '\n'
-      end)
-    lst;
+        Bytes.to_string s ^ "\r\n"
+      ) (List.filter (fun f -> f <> "") lst)
+  in
+  let utf16, lst =
+    match List.map toutf16 lst with
+    | lst ->
+        true, lst
+    | exception Not_utf8 ->
+        false, lst
+  in
+  if utf16 then output_string oc "\xFF\xFE"; (* LE BOM *)
+  List.iter (fun s -> output_string oc s) lst;
   close_out oc;
   "@" ^ responsefile
 
@@ -191,6 +258,15 @@ let find_file fn =
     | Some x -> Some x
     | None -> if !use_cygpath then find_file_in (cygpath l) else None
 
+let rec map_until_found f = function
+  | [] ->
+      None
+  | x::xs ->
+      match f x with
+      | None ->
+          map_until_found f xs
+      | r ->
+          r
 
 let find_file =
   let memo = Hashtbl.create 16 in
@@ -200,12 +276,16 @@ let find_file =
     with Not_found ->
       try Hashtbl.find memo (k ^ ".lib")
       with Not_found ->
-        let fn =
+        let fns =
           if String.length fn > 2 && String.sub fn 0 2 = "-l" then
-            "lib" ^ (String.sub fn 2 (String.length fn - 2))
-          else fn in
+            let base = String.sub fn 2 (String.length fn - 2) in
+            if !toolchain = `MSVC || !toolchain = `MSVC64 then
+              ["lib" ^ base; base]
+            else
+              ["lib" ^ base]
+          else [fn] in
         let r =
-          match find_file fn with
+          match map_until_found find_file fns with
           | Some fn -> fn
           | None ->
               failwith (Printf.sprintf "Cannot find file %S" fn)
@@ -917,6 +997,9 @@ let build_dll link_exe output_file files exts extra_args =
           else
             temp_file "dyndll_implib" ".lib"
         in
+        (* VS 2017.3 doesn't seem to be able to cope with /implib: existing but
+           being an empty file. *)
+        let c = open_out implib in output_string c "x"; close_out c;
         let _impexp = add_temp (Filename.chop_suffix implib ".lib" ^ ".exp") in
         let extra_args =
           if !custom_crt then "/nodefaultlib:LIBCMT /nodefaultlib:MSVCRT " ^ extra_args
